"""
Compiles the Irenic engine and minifies it (optional).

-c path_to_file to specify the configuration file. |Required
-o to obfuscate the final result. |Optional

Example:

python build.py -c config1.txt config2.txt -o -c config3.txt

This will make three files, all of them obfuscated.
"""

import sys
from subprocess import call

def build (configName, obfuscate):
    config = open(configName, "r").read() #Read the configuration file specified in the command line.

    paths = config.split("\n") #Array of all the paths to add into the build.
    paths = [path for path in paths if len(path) > 0] #Remove all empty characters caused by newlines.
    paths = [path for path in paths if path[0] != "#"] #Remove all comments.

    nameTo = paths[0] #Name of the destination file.
    prefix = paths[1] #This is added to every file.
    paths = paths[2:len(paths)] #This leaves only the paths in the array.

    #Add the prefix in front of all the files.
    if prefix != "-":
        for i in range(len(paths)):
            paths[i] = prefix + paths[i]

    toWrite = "" #The final build.

    #Iterate through all the files so they can be added to the build.
    for path in paths:
        toWrite += open(path, "r").read() #Add a file specified in the configuration to the build.
                
    open(nameTo, "w").write(toWrite) #Write the build to the specified file.test

    if obfuscate:
        call(["uglifyjs", "--overwrite", nameTo]);    


#Flags:
obfuscate = False #The user wants the final result to be obfuscated.
foundConfiguration = False #The user specified a configuration file.

configNames = []

#Find all the user-specified options in the comand line arguments.
for i in range(len(sys.argv) - 1):
    real = i + 1 #The first argument is the name of the script.
    
    if sys.argv[real] == "-c":
        for j in range(len(sys.argv) - real - 1):
            realj = real + j + 1
            
            if sys.argv[realj].startswith("-"):
                break;
            else:
                configNames.append(sys.argv[realj]) #The name of the configuration file.
                
                if not foundConfiguration:
                    foundConfiguration = True


    if sys.argv[real] == "-o":
        obfuscate = True
    

#The engine cannot be built if the config file isn't found.
if not foundConfiguration:
    print "\nError, you didn't specify the configuration file.\n"
    print "\"-c path_to_file\" to specify a file, without the quotations.\n"
    print "Configuration files tell the build script which files to include and how to name the final .js script.\n"
    sys.exit()


for configName in configNames:
    build(configName, obfuscate)

