Irenic.Particle = function(args) //image, velocity
{
    var particle = this;

    Irenic.Picture.call(particle, args);
    
    particle.velocity = args.velocity; //The Emitter will always supply a velocity so there's no need for default values.
    particle.angularVelocity = args.angularVelocity;
    particle.position = args.position; //The Emitter provides this as well.
    
    particle.start = new Date().getTime(); //So we know when to remove it and how to set the alpha.
}

Irenic.Particle.prototype = Object.create(Irenic.Picture.prototype);
Irenic.Particle.prototype.constructor = Irenic.Particle

Irenic.ParticleEmitter = function(args)
{
    var emitter = this;
    
    emitter.havent = 0;
    emitter.position = args.position;
    emitter.images = args.images; //These get fed to the particles.
    emitter.particles = [];
    emitter.last = new Date().getTime(); //The last time we updated.
    emitter.lastParticle = new Date().getTime(); //The last time a particle was created.
    emitter.emitting = args.emitting || true;
    emitter.width = 0; //Needed by the renderer.
    emitter.height = 0; //Needed by the renderer.
    
    //The default settings for the emitter, overwritten by [args.settings].
    emitter.settings = new Irenic.Settings(Irenic.defaultEmitterSettings);
    emitter.settings.applySettings(args.settings);    
    //console.log(emitter.settings);
}

Irenic.ParticleEmitter.prototype.update = function()
{
    var emitter = this;

    var timePassed = new Date().getTime() - emitter.last;
    timePassed *= 0.001; //Get the seconds.
    emitter.last = new Date().getTime();
    
    for (var i = 0; i < emitter.particles.length; i ++)
    {        
        emitter.particles[i].position.x += emitter.particles[i].velocity.x*timePassed;
        emitter.particles[i].position.y += emitter.particles[i].velocity.y*timePassed;

        emitter.particles[i].velocity.x += emitter.settings.acceleration.x*timePassed;
        emitter.particles[i].velocity.y += emitter.settings.acceleration.y*timePassed;

        if (emitter.settings.terminalVelocity)
        {
            var vel = emitter.particles[i].velocity.x*emitter.particles[i].velocity.x + emitter.particles[i].velocity.y*emitter.particles[i].velocity.y;
            
            if (vel*vel > emitter.settings.terminalVelocity)
            {
                if (!emitter.warned)
                {
                    emitter.warned = true;
                    console.warn("Particle terminal velocity not yet supported.");
                }
            }
        }
        
        emitter.particles[i].phi += emitter.particles[i].angularVelocity*timePassed;
        
        var life = new Date().getTime() - emitter.particles[i].start;
        
        if (life > emitter.settings.lifeTime)
        {
            emitter.particles.splice(i, 1);
            -- i;
        }
        else
        {        
            emitter.particles[i].alpha = emitter.settings.fade.call(emitter, life);
        }
    }
    
    //If new particles need to be created...
    if (new Date().getTime() - emitter.lastParticle > 1000/emitter.settings.speed)
    {
        var newParticles = Math.floor((new Date().getTime() - emitter.lastParticle)/(1000/emitter.settings.speed)); //How many of them?
        emitter.lastParticle = new Date().getTime(); //This is now the last time we created a particle.

        //If the emitter is emitting...
        if (emitter.emitting)
        {
            //Create newParticles particles.
            for(var j = 0; j < newParticles; j ++)
            {
                emitter.createParticle();
            }
        }
    }
}

Irenic.ParticleEmitter.prototype.draw = function(args)
{
    var emitter = this;

    //scene.position - (camera.position - renderer.wh/2)
    position = [args.position[0] - emitter.position.x, args.position[1] - emitter.position.y];

    for (var i = 0; i < emitter.particles.length; i ++)
    {
        particleArgs = 
        {
            "position": [emitter.particles[i].position.x + position[0], emitter.particles[i].position.y + position[1]], //We basically replaced the emitter's position with the particle's position.
            "context": args.context            
        }

        emitter.particles[i].draw(particleArgs);
    }
}

Irenic.ParticleEmitter.prototype.createParticle = function()
{
    var emitter = this;
    
    var whichImage = Math.floor(Math.random()*emitter.images.length);

    r = emitter.settings.velocity.getInt(); //The velocity is a range.

    var phi = emitter.settings.direction.getFloat();
    var x = Math.sin(phi)*r;    
    var y = -Math.cos(phi)*r;    
    
    var particleArgs =
    {
        "velocity": new Irenic.Vector2D({"x": x, "y": y}),
        "angularVelocity": emitter.settings.angularVelocity.getFloat(),
        "position": new Irenic.Vector2D({"x": emitter.position.x, "y": emitter.position.y}),
        "image": emitter.images[whichImage]
    };
         
    emitter.particles.push(new Irenic.Particle(particleArgs));
}

Irenic.ParticleEmitter.prototype.addImage = function(image)
{
    var emitter = this;

    emitter.images.push(image);
}
