//A collection of HTML5 images with functionality for animation.
Irenic.Animation = function(args)
{
    var animation = this;
    
    animation.images = args.images; //An array of HTML5 image objects.
    animation.phi = args.phi || 0; //Local phi.
    animation.alpha = args.alpha || 1; //Local alpha.
    animation.width = animation.images[0].width; //All frames are expected to be of the same width.
    animation.height = animation.images[0].height; //All frames are expected to be of the same width.
    animation.r = Math.sqrt(animation.width*animation.width + animation.height*animation.height)/2;
    animation.i = args.i || 0; //This denotes on which frame the animation is currently at.
    animation.FPS = args.FPS || 10; //Used by the update method, the number of frames the animation changes per second. The more the smoother.
    animation.last = new Date().getTime(); //Timestamp of the last framechange.
}

//Needs to get FPS as an argument and use it in update().
Irenic.Animation.prototype.draw = function(args)
{
    var animation = this;

    var position = args.position; //This is the correctly translated position according to the camera.
    
    var phi = args.phi || 0;
    phi += animation.phi;
    
    var alpha = args.alpha || 1;
    alpha *= animation.alpha;
    
    var context = args.context;
    var image = animation.getImage(); //The current frame.
        
    //Rendering.
    context.save(); //Save the context state, we're about to change it a lot.

    context.alpha *= alpha; //A scene might've already assigned an alpha, we would be messing with it if we just overrid it.          
    
    context.translate(position[0] + image.width/2, position[1] + image.height/2); //Translate the context to the center of the current frame.
    context.rotate(phi);
    //context.drawImage(image, Math.floor(-image.width/2) + 0.5, Math.floor(-image.height/2) + 0.5);
    context.drawImage(image, -image.width/2, -image.height/2);

    context.restore(); //Get the state back.
}

//Returns the current frame. Pretty much useless. animation.images[animation.i] is better and does the same thing (without the function overhead).
Irenic.Animation.prototype.getImage = function()
{
    return this.images[this.i];
}

Irenic.Animation.prototype.next = function()
{
    var animation = this;
    
    animation.i ++;
    
    if (animation.i > animation.images.length - 1)
    {
        animation.i = 0;
    }
}

Irenic.Animation.prototype.update = function()
{
    var animation = this;
    
    timePassed = new Date().getTime() - animation.last;
    
    if (timePassed > 1000/animation.FPS)
    {
        animation.last = new Date().getTime();
        animation.next();
    }
}

Irenic.Animation.prototype.getRect = function()
{
    var animation = this;

    return 
    [
        new Vector2D({"x": -animation.width/2, "y": -animation.height/2}),
        new Vector2D({"x": animation.width/2, "y": -animation.height/2}),
        new Vector2D({"x": animation.width/2, "y": animation.height/2}),
        new Vector2D({"x": -animation.width/2, "y": animation.height/2}),
    ];
}

