Irenic.keyCodes = 
{
    "w": 87,
    "a": 65,
    "s": 83,
    "d": 68,
    "e": 69,
    "one": 49,
    "ctrl": 17,
    "z": 90,
    "p": 107,
    "m": 109,
    "shift": 16
};

Irenic.InputHandler = function(args)
{
    var inputHandler = this;
    
    inputHandler.keys =
    {
	    "w": false,
	    "a": false,
	    "s": false,
	    "d": false,
	    "e": false,
	    "z": false,
	    "one": false,
	    "ctrl": false,
	    "plus": false,
	    "minus": false,
	    "shift": false,
    };
    
    inputHandler.mouse = 
    {
        "position": new Irenic.Vector2D({}),
        "lmb": false,
        "wheel": false,
        "rmb": false
    };
    
    inputHandler.events = [];
    inputHandler.listeningTo = [];

    inputHandler.onKeyDown = inputHandler.onKeyDown.bind(inputHandler);
    inputHandler.onKeyUp = inputHandler.onKeyUp.bind(inputHandler);
    inputHandler.onMouseMove = inputHandler.onMouseMove.bind(inputHandler);
    inputHandler.onMouseUp = inputHandler.onMouseUp.bind(inputHandler);
    inputHandler.onMouseDown = inputHandler.onMouseDown.bind(inputHandler);
    inputHandler.logListener = inputHandler.logListener.bind(inputHandler);
}


Irenic.InputHandler.prototype.onMouseMove = function(event)
{
    var inputHandler = this;

    inputHandler.mouse.position.x = event.clientX;    
    inputHandler.mouse.position.y = event.clientY;    
}

Irenic.InputHandler.prototype.onMouseDown = function(event)
{
    var inputHandler = this;

    switch (event.button)
    {
        case 0:
            inputHandler.mouse.lmb = true;
        break;           

        case 1:
            inputHandler.mouse.wheel = true;
        break;           

        case 2:
            inputHandler.mouse.rmb = true;
        break;           
    }
}

Irenic.InputHandler.prototype.onMouseUp = function(event)
{
    var inputHandler = this;

    switch (event.button)
    {
        case 0:
            inputHandler.mouse.lmb = false;
        break;           

        case 1:
            inputHandler.mouse.wheel = false;
        break;           

        case 2:
            inputHandler.mouse.rmb = false;
        break;           
    }
}


Irenic.InputHandler.prototype.onKeyDown = function(event)
{
    var inputHandler = this;

	var code = event.which || event.keyCode;

    //Iterate through all known keycodes...
    for (key in Irenic.keyCodes)
    {
        if (code == Irenic.keyCodes[key]) //If the pressed key's code is equal to the current one...
        {
            inputHandler.keys[key] = true; //That means that it was the one that was pressed, so set it to true.
        }
    }
}

Irenic.InputHandler.prototype.onKeyUp = function(event)
{
    var inputHandler = this;

	var code = event.which || event.keyCode;

    //Iterate through all known keycodes...
    for (key in Irenic.keyCodes)
    {
        if (code == Irenic.keyCodes[key]) //If the released key's code is equal to the current one...
        {
            inputHandler.keys[key] = false; //That means that it was the one that was released, so set it to false.
        }
    }
}

Irenic.InputHandler.prototype.attach = function()
{
    var inputHandler = this;

    document.addEventListener("keydown", inputHandler.onKeyDown, false);
    document.addEventListener("keyup", inputHandler.onKeyUp, false);
    document.addEventListener("mousemove", inputHandler.onMouseMove, false);
    document.addEventListener("mouseup", inputHandler.onMouseUp, false);
    document.addEventListener("mousedown", inputHandler.onMouseDown, false);
}

Irenic.InputHandler.prototype.detach = function()
{
    var inputHandler = this;

    document.removeEventListener("keydown", inputHandler.press, false);
    document.removeEventListener("keyup", inputHandler.release, false);
    document.removeEventListener("mousemove", inputHandler.onMouseMove, false);
    document.removeEventListener("mouseup", inputHandler.onMouseUp, false);
    document.removeEventListener("mousedown", inputHandler.onMouseDown, false);
}











Irenic.InputHandler.prototype.getEvents = function()
{
    var inputHandler = this;

    ret = inputHandler.events.slice(); //If we return the actual object, we're just returning a reference to it, so we need to copy it. 
    inputHandler.events = []; //Empty the log.
    
    return ret; //Returned the copied array.
}


Irenic.InputHandler.prototype.logListener = function(event)
{
    var inputHandler = this;
    
    inputHandler.events.push(event);
}


Irenic.InputHandler.prototype.listen = function(name)
{
    var inputHandler = this;
    
    if (name != "mousemove")
    {    
        document.addEventListener(name, inputHandler.logListener, false);
        inputHandler.listeningTo.push(name);
    }
    else
    {
        console.warn("Can't log mousemove, it'd make a mess.");
    }
}

Irenic.InputHandler.prototype.ignore = function(name)
{
    var inputHandler = this;

    document.addEventListener(name, inputHandler.logListener, false);
    
    for (var i = 0; i < inputHandler.listeningTo.length; i ++)
    {
        if (inputHandler.listeningTo[i] == name)
        {
            inputHandler.listeningTo.splice(i, 1);
            break; //Once we've removed the listener in question, there's no need to go on.
        }
    }
}

Irenic.InputHandler.prototype.ignoreAll = function()
{
    var inputHandler = this;

    for (var i = 0; i < inputHandler.listeningTo.length; i ++)
    {
        document.removeEventListener(name, inputHandler.logListener, false);
        inputHandler.listeningTo.splice(i, 1);
        -- i;
    }
}


Irenic.InputHandler.prototype.contextMenuBlock = function(event)
{
    event.preventDefault();
}


Irenic.InputHandler.prototype.noContextMenu = function()
{
    var inputHandler = this;

    document.addEventListener("contextmenu", inputHandler.contextMenuBlock, false);
}

Irenic.InputHandler.prototype.contextMenu = function()
{
    var inputHandler = this;

    document.removeEventListener("contextmenu", inputHandler.contextMenuBlock, false);
}

