Irenic.Update = function (args)
{
    var update = this;
    
    update.t = 1000/60 || args.t; //Interval.
    update.f = args.f; //The fire function to be executed once the update is ready.
    update.lastUpdated = args.lastUpdated || new Date().getTime(); //The time at which we last updated.
    update.timePassed = 0; //Time since we last updated.
    update.going = true;
    update.alive = true; //Used by the UpdateHandler to remove dead updates.
}

Irenic.Update.prototype.update = function(currentTime)
{
    var update = this;

    var currentTime = new Date().getTime();
    var timePassed = currentTime - update.lastUpdated;

    if (timePassed > update.t && update.going && update.alive)
    {
        update.lastUpdated = currentTime;
        
        return update.f(timePassed);
    }
    else
    {
        return false;
    }
}

Irenic.Update.prototype.pause = function()
{
    var update = this;

    update.going = true;    
    update.pausedAt = new Date().getTime();
}

Irenic.Update.prototype.go = function()
{
    var update = this;

    update.going = true;    
    
    //Pick up from where you left.
    var difference = update.pausedAt - update.lastUpdated
    update.lastUpdated = new Date().getTime() - difference;
}

Irenic.Update.prototype.stop = function()
{
    var update = this;
    
    update.alive = false; //Tell the handler to remove this.
}





//Just here so the updates can get out of the array peacefully.
Irenic.UpdateHandler = function(args)
{
    var handler = this;
    
    handler.updates = []; //The list of all updates.
    handler.lastUpdated = new Date().getTime();
}

Irenic.UpdateHandler.prototype.update = function(currentTime)
{
    var handler = this;

    var currentTime = currentTime || new Date().getTime();    
    var timePassed = currentTime - handler.lastUpdated;
    
    handler.lastUpdated = currentTime;
    
    for (var i = 0; i < handler.updates.length; i ++)
    {   
        handler.updates[i].update(currentTime); //Updates manage themselves.
        
        //If the update died...
        if (!handler.updates[i])
        {
            handler.updates.splice(i, 1); //Remove it. The garbage collector will take care of it.
            -- i;
        }
    }   
}

Irenic.UpdateHandler.prototype.add = function(update)
{
    var handler = this;
    
    handler.updates.push(update);
}
